superstrict


framework brl.blitz
import brl.glmax2d
import "common.bmx"
import "gui.bmx"




type TGame
	global levels_dir:string = "lvl"
	field timer:TTimer
	field button_continue:TButton
	field levels:TList
	field current_level:TLink
	field progress_file:string = "conf/progress"
	function comparelevelstrings:int(o1:object,o2:object)
		local s1:string = string(o1), s2:string = string(o2)
		if s1.startswith(levels_dir+"/level") then s1 = s1[9..s1.length]
		if s2.startswith(levels_dir+"/level") then s2 = s2[9..s2.length]
		local i1:int = int(s1), i2:int = int(s2)
		if i1 < i2 then return -1
		if i1 > i2 then return 1
		return 0
	endfunction
	method new()
		readlevels()
		if appargs.length >= 2 then step_to_level(appargs[1]) else load_progress()
		timer = TTimer.create(10)
		button_continue = TButton.create("Next",390,10)
	endmethod
	method readlevels()
		levels = new TList
		local dir:int = readdir(levels_dir)
		repeat
			local file:string = nextfile(dir)
			if file = "" then exit
			if file = "." or file = ".." or file.startswith("_") then continue
			levels.addlast(levels_dir+"/"+file)
		forever
		closedir(dir)
		levels.sort(true,comparelevelstrings)
		current_level = levels.firstlink()
	endmethod
	method step_to_level(lvl:string)
		while current_level <> null and string(current_level.value()) <> lvl
			current_level = current_level.nextlink()
		wend
	endmethod
	method save_progress()
		if current_level <> null then
			local stream:TStream = writefile(progress_file)
			stream.writeline(string(current_level.value()))
			stream.close()
		else
			if filetype(progress_file) = 1 then deletefile progress_file
		endif
	endmethod
	method load_progress()
		local stream:TStream = readfile(progress_file)
		if not stream then return
		local progress:string = stream.readline()
		stream.close()
		step_to_level(progress)
	endmethod
	method level_done:int(mx:int,my:int,mhit:int)
		button_continue.draw()
		return button_continue.clicked(mhit,mx,my)
	endmethod
	method ending_message()
		timer = TTimer.create(50)
		setcolor 255,255,255
		local scale:float = 200
		while scale > 1.001
			scale :- log(scale)/2
			timer.wait()
			cls
			setscale scale,scale
			drawtext "THE END",graphicswidth()/2-28*scale,graphicsheight()/2-8*scale
			flip
		wend
		delay 2000
	endmethod
	method main()
		while current_level <> null
			local current_file:string = string(current_level.value())
			gamefield = TGamefield.load_from_file(current_file)
			repeat
				timer.wait()
				local mx:int = mousex()
				local my:int = mousey()
				local mhit1:int = mousehit(1)
				local mhit2:int = mousehit(2)
				cls
				if appterminate() then
					save_progress()
					end
				endif
				if gamefield.finished() and level_done(mx,my,mhit1) then exit
				if mhit1 then gamefield.mouse(1,mx,my)
				if mhit2 then gamefield.mouse(2,mx,my)
				
				gamefield.run()
				gamefield.draw(10,50)
				setcolor 255,255,255
				drawtext current_file,10,10
				local text_wrap:string
				if gamefield.wrap_enabled then text_wrap = "Wrap: ON" else text_wrap = "Wrap: OFF"
				drawtext text_wrap,200,10
				flip
			forever
			current_level = current_level.nextlink()
		wend
		ending_message()
		save_progress()
	endmethod
endtype



(new TGame).main()

